import pytest

from instawow.resolvers import Defn


@pytest.mark.asyncio
async def test_finding_damaged_pkgs(mock_all, manager):
    molinari_defn = Defn(source='curse', name='molinari')
    molinari = (await manager.install([molinari_defn], False))[molinari_defn].pkg
    assert not manager.find_damaged_pkgs()
    (manager.config.addon_dir / molinari.folders[0].name).rename(manager.config.addon_dir / 'Foo')
    damaged_pkgs = manager.find_damaged_pkgs()
    assert molinari in damaged_pkgs



from aiohttp.test_utils import TestClient, TestServer
import pytest

from instawow.config import Config
from instawow.json_rpc_server import ErrorResponse, Request, SuccessResponse, create_app


@pytest.fixture
async def ws(full_config, monkeypatch):
    monkeypatch.setenv('INSTAWOW_CONFIG_DIR', str(full_config['config_dir']))
    app = await create_app()
    server = TestServer(app)

    async with TestClient(server) as client:
        async with client.ws_connect('/v0') as ws:
            yield ws


@pytest.mark.asyncio
async def test_write_config(request, full_config, ws):
    config_values = {**full_config, 'profile': request.node.name}
    rpc_request = Request(
        method='config.write',
        params={'values': config_values},
        id=request.node.name,
    )
    await ws.send_str(rpc_request.json())
    rpc_response = SuccessResponse.parse_raw(await ws.receive_str())
    assert rpc_response.id == request.node.name
    assert Config.parse_obj(rpc_response.result) == Config.parse_obj(config_values)


# @pytest.mark.asyncio
# async def test_infer_config(request, full_config, ws):
#     rpc_request = Request(
#         method='config.infer',
#         params={'values': {'profile': request.node.name, 'addon_dir': full_config['addon_dir']}},
#         id=request.node.name,
#     )
#     await ws.send_str(rpc_request.json())
#     rpc_response = SuccessResponse.parse_raw(await ws.receive_str())
#     assert rpc_response.result['profile'] == request.node.name


@pytest.mark.asyncio
async def test_install_with_invalid_params(request, ws):
    rpc_request = Request(
        method='install',
        params={},
        id=request.node.name,
    )
    await ws.send_str(rpc_request.json())
    rpc_response = ErrorResponse.parse_raw(await ws.receive_str())
    assert rpc_response.error['code'] == -32602


@pytest.mark.xfail
@pytest.mark.asyncio
async def test_install_with_uninitialised_profile(request, ws):
    rpc_request = Request(
        method='install',
        params={
            'profile': request.node.name,
            'defns': [{'source': 'curse', 'name': 'molinari'}],
            'replace': False,
        },
        id=request.node.name,
    )
    await ws.send_str(rpc_request.json())
    rpc_response = ErrorResponse.parse_raw(await ws.receive_str())
    assert rpc_response.error['code'] == -32001



import os

import nox

nox.options.envdir = '.py-nox'


@nox.session(python=['3.7', '3.8'])
def test(session):
    session.install('.[server,test]')
    session.run('coverage', 'run', '-m', 'pytest')
    session.run('coverage', 'report', '-m')


@nox.session(python=False)
def update_typeshed(session):
    types_dir = '.py-types'

    session.run('rm', '-rf', types_dir)
    session.run(
        *f'git clone --depth 1 https://github.com/python/typeshed {types_dir}/typeshed'.split()
    )


@nox.session(python=['3.7', '3.8'])
def type_check(session):
    session.install('.[server,test]')
    session.run('npx', '--cache', '.npm', 'pyright', '--lib')


@nox.session(python='3.7', reuse_venv=True)
def reformat(session):
    session.install('isort>=5.0.7', 'black>=19.10b0')
    for cmd in 'isort', 'black':
        session.run(cmd, 'instawow', 'tests', 'noxfile.py', 'setup.py')


@nox.session(python=False)
def clobber_build_artefacts(session):
    session.run('rm', '-rf', 'build', 'dist', 'instawow.egg-info')


@nox.session(python='3.7')
def build(session):
    clobber_build_artefacts(session)
    session.install('pep517')
    session.run('python', '-m', 'pep517.build', '.')


@nox.session(python='3.7')
def publish(session):
    session.install('twine')
    session.run('twine', 'upload', 'dist/*')


@nox.session
def nixify(session):
    nixify_dir = os.environ.get('INSTAWOW_NIXIFY_DIR', '.')

    session.cd(nixify_dir)
    # The latest published version of pypi2nix (2.0.4) overrides the pip
    # derivation's fetch URL with an old version of pip from GitHub which cannot
    # be built with an up-to-date derivation because the latter attempts
    # to apply a patch to a file which does not exist in the pip of olde
    session.install('pypi2nix @ https://github.com/nix-community/pypi2nix/archive/0dbd11.zip')
    session.run('pypi2nix', '-vvv', '-e', 'instawow')
